@@ -17,6 +17,7 @@
 # import hashlib
 import re
 import os
+import urllib.parse
 import utils.objectUtils as obj
 
 
@@ -207,6 +208,12 @@ def escape_regex(value):
     return value
 
 
+def strip_end(text, suffix):
+    if suffix and text.endswith(suffix):
+        return text[:-len(suffix)]
+    return text
+
+
 def file_path(value, **kwargs):
     '''
         Formats the path for use in windows.
@@ -220,8 +227,17 @@ def file_path(value, **kwargs):
 
         Keyword Arguments
         -------------------------
-        `escape_spaces` {bool}
+        [`strip_trailing_separator`=True] {bool}
+            if False, the path is allowed to end with a trailing separator.
+        [`escape_spaces`=False] {bool}
             if True, any segments of the path that contains a space is wrapped in quotes.
+        [`url`=False] {bool}
+            if True, the path seperator is set to "/" regardless of the os settings.
+        [`url_encode`=False] {bool}
+            If True the url is has all special characters encoded.
+        [`url_decode`=False] {bool}
+            If True the url is has all special characters decoded.
+
 
         Return {string}
         ----------------------
@@ -235,13 +251,19 @@ def file_path(value, **kwargs):
         `version`: 1.0
         `method_name`: file_path
     '''
+    strip_trailing_separator = obj.get_kwarg(["strip_trailing_separator"], True, (bool), **kwargs)
+    treat_as_url = obj.get_kwarg(["url"], False, (bool), **kwargs)
     escape_spaces = obj.get_kwarg(["escape spaces"], False, (bool), **kwargs)
     path_sep = escape_regex(os.path.sep)
+    if treat_as_url is True:
+        return url(value, **kwargs)
     # print(f"path_sep: {path_sep}")
     value = re.sub(r'(\\|\/)', path_sep, value)
     regex = re.compile(rf'{path_sep}+')
     value = re.sub(regex, path_sep, value)
-    value = clean_path(value)
+    # only clean the path for non urls
+    if len(value) > 250:
+        value = clean_path(value)
     if escape_spaces:
         path_array = value.split(os.path.sep)
         new_path_array = []
@@ -251,6 +273,58 @@ def file_path(value, **kwargs):
                 seg_string = f'"{seg_string}"'
             new_path_array.append(seg_string)
         value = path_sep.join(new_path_array)
+    if strip_trailing_separator is True:
+        value = strip_end(value, path_sep)
+    return value
+
+
+def url(value, **kwargs):
+    '''
+        Formats a url string
+
+        ----------
+
+        Arguments
+        -------------------------
+        `value` {str}
+            The url to format
+
+        Keyword Arguments
+        -------------------------
+        [`encode`=False] {bool}
+            If True the url is has all special characters encoded.
+        [`decode`=False] {bool}
+            If True the url is has all special characters decoded.
+        [`strip_trailing_separator`=True] {bool}
+            if False, the path is allowed to end with a trailing separator.        
+
+        Return {str}
+        ----------------------
+        The formatted url
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 12-17-2021 14:28:23
+        `memberOf`: string_format
+        `version`: 1.0
+        `method_name`: url
+    '''
+    strip_trailing_separator = obj.get_kwarg(["strip_trailing_separator"], True, (bool), **kwargs)
+    encode = obj.get_kwarg(["encode", "url_encode"], False, (bool), **kwargs)
+    decode = obj.get_kwarg(["decode", "url_decode"], False, (bool), **kwargs)
+    path_sep = "/"
+    value = re.sub(r'(\\|\/)', path_sep, value)
+    regex = re.compile(rf'{path_sep}+')
+    value = re.sub(regex, path_sep, value)
+    if encode is True:
+        value = urllib.parse.quote(value)
+    if decode is True:
+        value = urllib.parse.unquote(value)
+
+    if strip_trailing_separator is True:
+        value = strip_end(value, path_sep)
+
     return value
 
 
